@@ -5,7 +5,7 @@ Copyright (C) Xavier Roche and other contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
+as published by the Free Software Foundation; either version 3
 of the License, or any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -48,16 +48,13 @@ extern "C" {
 #include <string.h>
 #include <time.h>
 
-#ifdef HAVE_UNISTD_H
+#ifndef _WIN32
 #include <unistd.h>
 #endif
-#if ( defined(_WIN32) ||defined(HAVE_SYS_TYPES_H) )
 #include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
-#endif
-#ifdef HAVE_DLFCN_H
+
+#ifndef _WIN32
 #include <dlfcn.h>
 #endif
 
