@@ -147,7 +147,7 @@ void cache_mayadd(httrackp* opt,cache_back* cache,htsblk* r,const char* url_adr,
           // stocker fichiers (et robots.txt)
           if ( url_save == NULL || (strnotempty(url_save)) || (strcmp(url_fil,"/robots.txt")==0)) {
             // ajouter le fichier au cache
-						cache_add(opt,cache,r,url_adr,url_fil,url_save,opt->all_in_cache,StringBuff(opt->path_html));
+						cache_add(opt,cache,r,url_adr,url_fil,url_save,opt->all_in_cache,StringBuff(opt->path_html_utf8));
             //
             // store a reference NOT to redo the same test zillions of times!
             // (problem reported by Lars Clausen)
@@ -367,9 +367,9 @@ void cache_add(httrackp* opt,cache_back* cache,const htsblk *r,const char* url_a
     } else {
       FILE* fp;
       // On recopie le fichier->.
-      off_t file_size=fsize(fconv(catbuff, url_save));
+      off_t file_size=fsize_utf8(fconv(catbuff, url_save));
       if (file_size>=0) {
-        fp=fopen(fconv(catbuff, url_save),"rb");
+        fp=FOPEN(fconv(catbuff, url_save),"rb");
         if (fp!=NULL) {
           char BIGSTK buff[32768];
           size_t nl;
@@ -498,10 +498,10 @@ void cache_add(httrackp* opt,cache_back* cache,const htsblk *r,char* url_adr,cha
       } else {  // recopier fichier dans cache
         FILE* fp;
         // On recopie le fichier->.
-        off_t file_size=fsize(fconv(catbuff, url_save));
+        off_t file_size=fsize_utf8(fconv(catbuff, url_save));
         if (file_size>=0) {
           if (cache_wLLint(cache_dat,file_size)!=-1) {
-            fp=fopen(fconv(catbuff, url_save),"rb");
+            fp=FOPEN(fconv(catbuff, url_save),"rb");
             if (fp!=NULL) {
               char BIGSTK buff[32768];
               ssize_t nl;
@@ -586,7 +586,7 @@ htsblk cache_readex(httrackp* opt,cache_back* cache,const char* adr,const char*
 
 // lecture d'un fichier dans le cache
 // si save==null alors test unqiquement
-static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,const char* fil,const char* save,char* location,
+static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,const char* fil,const char* target_save,char* location,
                                char* return_save, int readonly) {
   char BIGSTK location_default[HTS_URLMAXSIZE*2];
   char BIGSTK buff[HTS_URLMAXSIZE*2];
@@ -678,9 +678,9 @@ static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,c
 
 					/* Previous entry */
 					if (previous_save_[0] != '\0') {
-						int pathLen = (int) strlen(StringBuff(opt->path_html));
-						if (pathLen != 0 && strncmp(previous_save_, StringBuff(opt->path_html), pathLen) != 0) {			// old (<3.40) buggy format
-							sprintf(previous_save, "%s%s", StringBuff(opt->path_html), previous_save_);
+						int pathLen = (int) strlen(StringBuff(opt->path_html_utf8));
+						if (pathLen != 0 && strncmp(previous_save_, StringBuff(opt->path_html_utf8), pathLen) != 0) {			// old (<3.40) buggy format
+							sprintf(previous_save, "%s%s", StringBuff(opt->path_html_utf8), previous_save_);
 						} else {
 							strcpy(previous_save, previous_save_);
 						}
@@ -694,49 +694,111 @@ static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,c
           r.adr=NULL;
           r.out=NULL;
           r.fp=NULL;
-          
-          if (save != NULL) {     /* ne pas lire uniquement header */
-            int ok = 0;
-                       
-#if HTS_DIRECTDISK
-            // Court-circuit:
-            // Peut-on stocker le fichier directement sur disque?
-            if (ok) {
-              if (r.msg[0] == '\0') {
-                strcpybuff(r.msg,"Cache Read Error : Unexpected error");
-              }
-            }
-            else if (!readonly && r.statuscode==HTTP_OK && !is_hypertext_mime(opt,r.contenttype, fil) && strnotempty(save)) {    // pas HTML, crire sur disk directement
-              r.is_write=1;    // crire
 
-              if (!dataincache) {
-								if (fexist(fconv(catbuff, save))) {  // un fichier existe dja
-									//if (fsize(fconv(save))==r.size) {  // mme taille -- NON tant pis (taille mal declaree)
-									ok=1;    // plus rien  faire
-									filenote(&opt->state.strc,save,NULL);        // noter comme connu
-									file_notify(opt,adr, fil, save, 0, 0, 1);        // data in cache
-								}
-							}
-              
-              if (!dataincache && !ok) { // Pas de donne en cache et fichier introuvable : erreur!
-                if (opt->norecatch) {
-                  file_notify(opt,adr, fil, save, 1, 0, 0);
-                  filecreateempty(&opt->state.strc, save);
-                  //
+          /* Do not get data ? Do some final tests. */
+          if (target_save == NULL) {
+            // si save==null, ne rien charger (juste en tte)
+            if (r.statuscode==HTTP_OK && !is_hypertext_mime(opt, r.contenttype, fil)) {    // pas HTML, crire sur disk directement
+              r.is_write = 1;  /* supposed to be on disk (informational) */
+            }
+
+            /* Ensure the file is present, because returning a reference to a missing file is useless! */
+            if (!dataincache) {   /* Data are supposed to be on disk */
+              if (!fexist_utf8(fconv(catbuff, previous_save))) {  // un fichier existe dja
+                if (!opt->norecatch) {
+                  if ((opt->debug>0) && (opt->log!=NULL)) {
+                    HTS_LOG(opt,LOG_DEBUG); fprintf(opt->log,"Cache: could not find %s"LF, previous_save);
+                  }
                   r.statuscode=STATUSCODE_INVALID;
-                  strcpybuff(r.msg,"File deleted by user not recaught");
-                  ok=1;     // ne pas rcuprer (et pas d'erreur)
-                } else {
-                  // ????? file_notify(opt,adr, fil, save, 1, 1, 0);
-                  r.statuscode=STATUSCODE_INVALID;
-                  strcpybuff(r.msg,"Previous cache file not found");
-                  ok=1;    // ne pas rcuprer
+                  strcpybuff(r.msg, "Previous cache file not found");
                 }
               }
-              
-              if (!ok) {        // load from cache
-                file_notify(opt,adr, fil, save, 1, 1, 1);        // data in cache
-                r.out=filecreate(&opt->state.strc, save);
+            } // otherwise, the ZIP file is supposed to be consistent with data.
+          }
+          /* Read data ? */
+          else {     /* ne pas lire uniquement header */
+            int ok = 0;
+
+#if HTS_DIRECTDISK
+            // Not ro, and pure data (not HTML and friends) to be saved now.
+            if (!readonly && r.statuscode==HTTP_OK && !is_hypertext_mime(opt,r.contenttype, fil) && strnotempty(target_save)) {    // pas HTML, crire sur disk directement
+              r.is_write=1;    // crire
+
+              // Data is supposed to be on disk
+              if (!dataincache) {
+                r.msg[0] = '\0';
+
+                // File exists on disk with declared cache name (this is expected!)
+                if (fexist_utf8(fconv(catbuff, previous_save))) {  // un fichier existe dja
+                  // Expected size ?
+                  const size_t fsize = fsize_utf8(fconv(catbuff, previous_save));
+                  if (fsize == r.size) {
+                    // Target name is the previous name, and the file looks good: nothing to do!
+                    if (strcmp(previous_save, target_save) == 0) {
+                      // So far so good
+                      ok=1;    // plus rien  faire
+                    }
+                    // Different filenames: rename now!
+                    else {
+                      char catbuff2[CATBUFF_SIZE];
+                      if (RENAME(fconv(catbuff, previous_save), fconv(catbuff2, target_save)) == 0) {
+                        // So far so good
+                        ok=1;    // plus rien  faire
+
+                        if ((opt->debug>1) && (opt->log!=NULL)) {
+                          HTS_LOG(opt,LOG_DEBUG); fprintf(opt->log, "File '%s' has been renamed since last mirror to '%s' ; applying changes"LF, previous_save, target_save); test_flush;
+                        }
+                      } else {
+                        r.statuscode=STATUSCODE_INVALID;
+                        strcpybuff(r.msg, "Unable to rename file on disk");
+                      }
+                    }
+                  } else {
+                    HTS_LOG(opt,LOG_WARNING); fprintf(opt->log, "warning: file size on disk ("LLintP") does not have the expected size ("LLintP"))"LF, (LLint) fsize, (LLint) r.size);
+                  }
+                }
+                // File exists with the target name and not previous one ?
+                // Suppose a broken mirror, with a file being renamed: OK
+                else if (fexist_utf8(fconv(catbuff, target_save))) {
+                  // Expected size ?
+                  const size_t fsize = fsize_utf8(fconv(catbuff, target_save));
+                  if (fsize == r.size) {
+                    // So far so good
+                    ok=1;    // plus rien  faire
+                  } else {
+                    HTS_LOG(opt,LOG_WARNING); fprintf(opt->log, "warning: renamed file size on disk ("LLintP") does not have the expected size ("LLintP"))"LF, (LLint) fsize, (LLint) r.size);
+                  }
+                }
+
+                // File is present and sane ?
+                if (ok) {
+                  // Register file not to wipe it later
+                  filenote(&opt->state.strc,target_save,NULL);        // noter comme connu
+                  file_notify(opt,adr, fil, target_save, 0, 0, 1);        // data in cache
+                }
+                // Nope
+                else {
+                  // Default behavior
+                  if (!opt->norecatch) {
+                    r.statuscode=STATUSCODE_INVALID;
+                    if (r.msg[0] == '\0') {
+                      strcpybuff(r.msg, "Previous cache file not found");
+                    }
+                  }
+                  // Do not recatch broken/erased files
+                  else {
+                    file_notify(opt,adr, fil, target_save, 1, 0, 0);
+                    filecreateempty(&opt->state.strc, target_save);
+                    //
+                    r.statuscode=STATUSCODE_INVALID;
+                    strcpybuff(r.msg, "File deleted by user not recaught");
+                  }
+                }
+              }
+              // Load data from cache (not from a disk file)
+              else {
+                file_notify(opt,adr, fil, target_save, 1, 1, 1);        // data in cache
+                r.out=filecreate(&opt->state.strc, target_save);
 #if HDEBUG
                 printf("direct-disk: %s\n",save);
 #endif
@@ -761,7 +823,7 @@ static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,c
                   fclose(r.out);
                   r.out=NULL;
 #ifndef _WIN32
-                  chmod(save,HTS_ACCESS_FILE);      
+                  chmod(target_save, HTS_ACCESS_FILE);      
 #endif          
                 } else {
                   r.statuscode=STATUSCODE_INVALID;
@@ -773,9 +835,11 @@ static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,c
             } else
 #endif
             { // lire en mmoire
-              
+            
+              // We need to get bytes on memory, but the previous version is (supposed to be) on disk.
               if (!dataincache) {
-                if (strnotempty(save)) { // Pas de donne en cache, bizarre car html!!!
+                // Empty previous save name, or file does not exist ?
+                if (!strnotempty(previous_save) || !fexist_utf8(previous_save)) { // Pas de donne en cache, bizarre car html!!!
                   // Hack: if error page data is missing (pre-3.45 releases), create one. We won't use it anyway.
                   if (!HTTP_IS_OK(r.statuscode)) {
                     const int size = 512;
@@ -794,31 +858,31 @@ static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,c
                   // Otherwise, this is a real error.
                   else {
                     r.statuscode=STATUSCODE_INVALID;
-                    strcpybuff(r.msg,"Previous cache file not found (2)");
+                    strcpybuff(r.msg,"Previous cache file not found (empty filename)");
                   }
-                } else {                 /* Read in memory from cache */
-                  if (strnotempty(previous_save) && fexist(previous_save)) {
-                    FILE* fp = fopen(fconv(catbuff, previous_save), "rb");
-                    if (fp != NULL) {
-                      r.adr = (char*) malloct((int) r.size + 4);
-                      if (r.adr != NULL) {
-                        if (r.size > 0 && fread(r.adr, 1, (int) r.size, fp) != r.size) {
-                          int last_errno = errno;
-                          r.statuscode=STATUSCODE_INVALID;
-                          sprintf(r.msg,"Read error in cache disk data: %s", strerror(last_errno));
-                        }
-                      } else {
+                } else {                 /* Read in memory from disk */
+                  FILE*const fp = FOPEN(fconv(catbuff, previous_save), "rb");
+                  if (fp != NULL) {
+                    r.adr = (char*) malloct((int) r.size + 4);
+                    if (r.adr != NULL) {
+                      if (r.size > 0 && fread(r.adr, 1, (int) r.size, fp) != r.size) {
+                        int last_errno = errno;
                         r.statuscode=STATUSCODE_INVALID;
-                        strcpybuff(r.msg,"Read error (memory exhausted) from cache");
+                        sprintf(r.msg,"Read error in cache disk data: %s", strerror(last_errno));
                       }
-                      fclose(fp);
+                    } else {
+                      r.statuscode=STATUSCODE_INVALID;
+                      strcpybuff(r.msg,"Read error (memory exhausted) from cache");
                     }
+                    fclose(fp);
                   } else {
                     r.statuscode=STATUSCODE_INVALID;
-                    strcpybuff(r.msg,"Cache file not found on disk");
+                    strcpybuff(r.msg,"Read error (unable to open disk file) from cache");
                   }
                 }
-              } else {
+              }
+              // Data in cache.
+              else {
                 // lire fichier (d'un coup)
                 r.adr = (char*) malloct((int) r.size+4);
                 if (r.adr!=NULL) {
@@ -837,10 +901,6 @@ static htsblk cache_readex_new(httrackp* opt,cache_back* cache,const char* adr,c
               }
             }
           }
-          // si save==null, ne rien charger (juste en tte)
-          else if (r.statuscode==HTTP_OK && !is_hypertext_mime(opt,r.contenttype, fil)) {    // pas HTML, crire sur disk directement
-            r.is_write = 1;  /* supposed to be on disk (informational) */
-          }
 
         } else {
           r.statuscode=STATUSCODE_INVALID;
@@ -1016,8 +1076,8 @@ static htsblk cache_readex_old(httrackp* opt,cache_back* cache,const char* adr,c
               int ok=0;
               
               r.is_write=1;    // crire
-              if (fexist(fconv(catbuff, save))) {  // un fichier existe dja
-                //if (fsize(fconv(save))==r.size) {  // mme taille -- NON tant pis (taille mal declaree)
+              if (fexist_utf8(fconv(catbuff, save))) {  // un fichier existe dja
+                //if (fsize_utf8(fconv(save))==r.size) {  // mme taille -- NON tant pis (taille mal declaree)
                 ok=1;    // plus rien  faire
                 filenote(&opt->state.strc,save,NULL);        // noter comme connu
                 file_notify(opt,adr, fil, save, 0, 0, 0);
@@ -1082,8 +1142,8 @@ static htsblk cache_readex_old(httrackp* opt,cache_back* cache,const char* adr,c
                   r.statuscode=STATUSCODE_INVALID;
                   strcpybuff(r.msg,"Previous cache file not found (2)");
                 } else {                 /* Read in memory from cache */
-                  if (strnotempty(return_save) && fexist(return_save)) {
-                    FILE* fp = fopen(fconv(catbuff, return_save), "rb");
+                  if (strnotempty(return_save) && fexist_utf8(return_save)) {
+                    FILE* fp = FOPEN(fconv(catbuff, return_save), "rb");
                     if (fp != NULL) {
                       r.adr = (char*) malloct((size_t)r.size + 4);
                       if (r.adr != NULL) {
@@ -1685,10 +1745,12 @@ void cache_init(cache_back* cache,httrackp* opt) {
 
 
 // lire un fichier.. (compatible \0)
+/* Note: NOT utf-8 */
 char* readfile(char* fil) {
   return readfile2(fil, NULL);
 }
 
+/* Note: NOT utf-8 */
 char* readfile2(char* fil, LLint* size) {
   char* adr=NULL;
 	char catbuff[CATBUFF_SIZE];
@@ -1714,6 +1776,30 @@ char* readfile2(char* fil, LLint* size) {
   return adr;
 }
 
+/* Note: utf-8 */
+char* readfile_utf8(char* fil) {
+  char* adr=NULL;
+	char catbuff[CATBUFF_SIZE];
+  const off_t len = fsize_utf8(fil);
+  if (len >= 0) {  // exists
+    FILE*const fp = FOPEN(fconv(catbuff, fil),"rb");
+    if (fp!=NULL) {  // n'existe pas (!)
+      adr = (char*) malloct(len+1);
+      if (adr!=NULL) {
+        if (len > 0 && fread(adr,1,len,fp) != len) {    // fichier endommag ?
+          freet(adr);
+          adr=NULL;
+        } else {
+          adr[len] = '\0';
+        }
+      }
+      fclose(fp);
+    }
+  }
+  return adr;
+}
+
+/* Note: NOT utf-8 */
 char* readfile_or(char* fil,char* defaultdata) {
   char* realfile=fil;
   char* ret;
