@@ -340,6 +340,7 @@ void usercommand_exe(const char* cmd,const char* file);
 int filters_init(char*** ptrfilters, int maxfilter, int filterinc);
 #ifndef HTTRACK_DEFLIB
 HTSEXT_API int structcheck(const char* path);
+HTSEXT_API int structcheck_utf8(const char* path);
 HTSEXT_API int dir_exists(const char* path);
 #endif
 HTS_INLINE int fspc(httrackp *opt,FILE* fp,const char* type);
@@ -347,6 +348,7 @@ char* next_token(char* p,int flag);
 //
 char* readfile(char* fil);
 char* readfile2(char* fil, LLint* size);
+char* readfile_utf8(char* fil);
 char* readfile_or(char* fil,char* defaultdata);
 #if 0
 void check_rate(TStamp stat_timestart,int maxrate);
