@@ -5,7 +5,7 @@ Copyright (C) Xavier Roche and other contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
+as published by the Free Software Foundation; either version 3
 of the License, or any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -263,7 +263,7 @@ void hash_write(hash_struct* hash,int lpos,int normalized) {
   else {
     printf("* hash_write=0!!\n");
     abortLogFmt("unexpected error in hash_write (pos=%d)" _ pos);
-    exit(1);
+    abort();
   }
 #endif
   //
