@@ -264,12 +264,12 @@ extern FILE* hts_dgb_(void);
 } while(0)
 
 // fonctions principales
-int http_fopen(httrackp *opt,char* adr,char* fil,htsblk* retour);
-int http_xfopen(httrackp *opt,int mode,int treat,int waitconnect,char* xsend,char* adr,char* fil,htsblk* retour);
+T_SOC http_fopen(httrackp *opt,char* adr,char* fil,htsblk* retour);
+T_SOC http_xfopen(httrackp *opt,int mode,int treat,int waitconnect,char* xsend,char* adr,char* fil,htsblk* retour);
 int http_sendhead(httrackp *opt,t_cookie* cookie,int mode,char* xsend,char* adr,char* fil,char* referer_adr,char* referer_fil,htsblk* retour);
 htsblk httpget(httrackp *opt,char* url);
 //int newhttp(char* iadr,char* err=NULL);
-int newhttp(httrackp *opt,const char* iadr,htsblk* retour,int port,int waitconnect);
+T_SOC newhttp(httrackp *opt,const char* iadr,htsblk* retour,int port,int waitconnect);
 HTS_INLINE void deletehttp(htsblk* r);
 HTS_INLINE int  deleteaddr(htsblk* r);
 HTS_INLINE void deletesoc(T_SOC soc);
@@ -337,7 +337,7 @@ void rawlinput(FILE* fp,char* s,int max);
 char* strstrcase(char *s,char *o);
 int ident_url_absolute(const char* url,char* adr,char* fil);
 void fil_simplifie(char* f);
-int is_unicode_utf8(unsigned char* buffer, unsigned int size);
+int is_unicode_utf8(const unsigned char* buffer, size_t size);
 void map_characters(unsigned char* buffer, unsigned int size, unsigned int* map);
 int ishtml(httrackp *opt,const char* urlfil);
 int ishtml_ext(const char* a);
@@ -411,9 +411,11 @@ int sig_ignore_flag( int setflag );     // flag ignore
 
 void cut_path(char* fullpath,char* path,char* pname);
 int fexist(const char* s);
+int fexist_utf8(const char* s);
 /*LLint fsize(const char* s);    */
 off_t fpsize(FILE* fp);
 off_t fsize(const char* s);    
+off_t fsize_utf8(const char* s);
 /* root dir */
 #ifndef HTTRACK_DEFLIB
 HTSEXT_API char* hts_rootdir(char* file);
@@ -488,6 +490,37 @@ void *hts_get_callback(t_hts_htmlcheck_callbacks *callbacks, const char *name);
 )
 */
 
+/* UTF-8 aware FILE API */
+#ifndef HTS_DEF_FILEAPI
+#ifdef _WIN32
+#define FOPEN hts_fopen_utf8
+HTSEXT_API FILE* hts_fopen_utf8(const char *path, const char *mode);
+#define STAT hts_stat_utf8
+typedef struct _stat STRUCT_STAT;
+HTSEXT_API int hts_stat_utf8(const char *path, STRUCT_STAT *buf);
+#define UNLINK hts_unlink_utf8
+HTSEXT_API int hts_unlink_utf8(const char *pathname);
+#define RENAME hts_rename_utf8
+HTSEXT_API int hts_rename_utf8(const char *oldpath, const char *newpath);
+#define MKDIR(F) hts_mkdir_utf8(F)
+HTSEXT_API int hts_mkdir_utf8(const char *pathname);
+#define UTIME(A,B) hts_utime_utf8(A,B)
+typedef struct _utimbuf STRUCT_UTIMBUF;
+HTSEXT_API int hts_utime_utf8(const char *filename, const STRUCT_UTIMBUF *times);
+#else
+/* The underlying filesystem charset is supposed to be UTF-8 */
+#define FOPEN fopen
+#define STAT stat
+typedef struct stat STRUCT_STAT;
+#define UNLINK unlink
+#define RENAME rename
+#define MKDIR(F) mkdir(F, HTS_ACCESS_FOLDER)
+typedef struct utimbuf STRUCT_UTIMBUF;
+#define UTIME(A,B) utime(A,B)
+#endif
+#define HTS_DEF_FILEAPI
+#endif
+
 #endif    // internals
 
 #undef PATH_SEPARATOR
