@@ -5,7 +5,7 @@ Copyright (C) Xavier Roche and other contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
+as published by the Free Software Foundation; either version 3
 of the License, or any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -31,7 +31,7 @@ Please visit our Website: http://www.httrack.com
 
 /* ------------------------------------------------------------ */
 /* File: htsshow.c console progress info                        */
-/* Only used on Linux version                                   */
+/* Only used on Linux & FreeBSD versions                        */
 /* Author: Xavier Roche                                         */
 /* ------------------------------------------------------------ */
 
@@ -163,6 +163,7 @@ static int use_show;
 static httrackp *global_opt = NULL;
 
 static void signal_handlers(void);
+static void signal_restore_ctl_z(void);
 
 int main(int argc, char **argv) {
   int ret = 0;
@@ -706,8 +707,15 @@ static void sig_ask( int code ) {        // demander
 #else
 static void sig_doback(int blind);
 static void sig_back( int code ) {       // ignorer et mettre en backing 
-  signal(code,sig_ignore);
-  sig_doback(0);
+  if (global_opt != NULL && !global_opt->background_on_suspend) {
+    signal( SIGTSTP , SIG_DFL); // ^Z
+    printf("\nInterrupting the program.\n"); fflush(stdout);
+    kill(getpid(), SIGTSTP);
+  } else {
+    // Background the process.
+    signal(code, sig_ignore);
+    sig_doback(0);
+  }
 }
 static void sig_ask( int code ) {        // demander
   char s[256];
@@ -752,11 +760,9 @@ static void sig_doback(int blind) {       // mettre en backing
     out = open("hts-nohup.out",O_CREAT|O_WRONLY,S_IRUSR|S_IWUSR);
   if (out == -1)
     out = open("/dev/null",O_WRONLY,S_IRUSR|S_IWUSR);
-  close(0);
-  close(1);
-  dup(out);
-  close(2);
-  dup(out);
+  dup2(out, 0);
+  dup2(out, 1);
+  dup2(out, 2);
   //
   switch (fork()) {
   case 0: 
@@ -765,7 +771,6 @@ static void sig_doback(int blind) {       // mettre en backing
     fprintf(stderr,"Error: can not fork process\n");
     break;
   default:            // pere
-    usleep(100000);   // pause 1/10s "A  microsecond  is  .000001s"
     _exit(0);
     break;  
 	}
@@ -773,11 +778,15 @@ static void sig_doback(int blind) {       // mettre en backing
 #endif
 
 static void sig_leave( int code ) {
-  signal(code, sig_term);  // quitter si encore
-  printf("\n** Finishing pending transfers.. press again ^C to quit.\n");
-  if (global_opt != NULL) {
-    // ask for stop
-    global_opt->state.stop=1;
+  if (global_opt != NULL && global_opt->state._hts_in_mirror) {
+    signal(code, sig_term);  // quitter si encore
+    printf("\n** Finishing pending transfers.. press again ^C to quit.\n");
+    if (global_opt != NULL) {
+      // ask for stop
+      global_opt->state.stop=1;
+    }
+  } else {
+    sig_term(code);
   }
 }
 
@@ -795,7 +804,7 @@ static void signal_handlers(void) {
 #if 0	/* BUG366763 */
 	signal( SIGHUP  , sig_back   );   // close window
 #endif
-	signal( SIGTSTP , sig_back   );   // ^Z
+	signal( SIGTSTP , sig_back );     // ^Z
 	signal( SIGTERM , sig_finish );   // kill <process>
 #if 0	/* BUG366763 */
 	signal( SIGINT  , sig_ask    );   // ^C
@@ -807,4 +816,10 @@ static void signal_handlers(void) {
 #endif
 }
 
+static void signal_restore_ctl_z(void) {
+#ifndef _WIN32
+  signal( SIGTSTP , SIG_DFL); // ^Z
+#endif
+}
+
 // fin routines de dtournement de SIGHUP & co
