@@ -12,7 +12,7 @@
 #include <stdlib.h>
 #else
 #include <stdio.h>
-//#include "celib.h"
+#include "celib.h"
 #endif
 #include <string.h>
 #include <time.h>
